function [im_dnois]=WGSM_Precip_Filt(im,Nsc,sig_w,nhood)
% This file encodes filtering of measurement error from a remotely sensed
% precipitation image using GSM-Wiener. log-normal prior density is assumed 
% for the multiplier process.
% im: the images
% Nsc: number of decomposition levels
% sig_w: variance of the measurment error(white gaussin noise)
% nhood: size of the neighborhood
% Image padding for construction of the pyramids using SWT, size~2^Nsc
% By: M-Ebtehaj Jan-2011, Rev00

pad_size1=2^Nsc*ceil(size(im)/2^Nsc)-size(im);
im_ns=padarray(im,pad_size1,'symmetric','post');
noise=sig_w*randn(size(im_ns));

% This following pyramids include "Nsc" levels of undecimated H-V-D subbands
% Consecutively from bottom-to-top
pyr_im=swt2(im_ns,Nsc,'haar');
pyr_nos=swt2(noise,Nsc,'haar');

% checking maximum magnitude of noise to perform GSM recovery
sig_pyr=zeros(1,3*Nsc);
for i=1:3*Nsc
    temp01=pyr_im(:,:,i);
    sig_pyr(i)=std2(temp01(temp01>0));
end
sig_max=min(sig_pyr);

if sig_w<0.9*sig_max
    % Performing GSM Filtering
    % Padding the pyramid to handle nlfilering boundary issues
    pad_size2=(nhood-1)/2*ones(1,2);
    pyr_im=padarray(pyr_im,pad_size2,'symmetric','both');
    pyr_nos=padarray(pyr_nos,pad_size2,'symmetric','both');
    % subband Filtering
    swt2_rec=zeros([size(im_ns) 3*Nsc+1]);
    h = waitbar(0,'Please wait...');
    for i=1:3*Nsc
        I=pyr_im(:,:,i:i+1);
        noise=pyr_nos(:,:,i:i+1);
        if mod(i,Nsc)==0,prnt=0;
        else
            prnt=1;
        end
        swt2_rec(:,:,i)=wiener_gsm(I,noise,nhood,prnt);
        perc=ceil(100*i/(3*Nsc));
        waitbar(i/(3*Nsc),h,['Rain-Gauge Error Filtering...','(',num2str(perc),'%)'])
    end   
    
    % Returning the pyramid into its size before non-linear filtering
    % and inserting the low pass subband into the filtered pyramid
    lim_r=pad_size2(1)+1:size(pyr_im,1)-pad_size2(1);
    lim_c=pad_size2(2)+1:size(pyr_im,2)-pad_size2(2);
    pyr_im=pyr_im(lim_r,lim_c,:);
    swt2_rec(:,:,3*Nsc+1)=pyr_im(:,:,3*Nsc+1);
    
    % Reconstruction of the image
    im_dnois = iswt2(swt2_rec,'haar');
    
    % Returning back the Pyramid to its original size, removing the dyadic
    % padding requirements
    im_dnois=im_dnois(1:size(im_dnois,1)-pad_size1(1),1:size(im_dnois,2)-pad_size1(2));
    im_dnois(im_dnois<0)=0;
else
    
    % Performing ordinary wiener Filtering
    disp('Warning: The magnitude of noise exceeds minimum subband variability!! Ordinary Wiener Filter is being used!!!')
    im_dnois=wiener2(im, nhood.*[1 1], sig_w);
    disp('100% is Done!')
end
close(h)    

