function [im_fused]=WGSM_TR_NEX_Proj(im_nex,im_trm,Nsc,sig_w,nhood_l,option)
% This file encodes filtering of measurement error from a remotely sensed
% precipitation image using GSM filtering. log-normal prior density is assumed
% for the multiplier process
% density for the multiplier
% im_nex: the NEXRAD images
% im_trm: the TRMM images
% Nsc: number of decomposition levels
% sig_w: variance of the measurment error(white gaussin noise)
% nhood_l: size of the neighborhood (odd) for low_pass projection
% option: 0: Incorporate envlope of observations; 1: Ordinary fusion
% By: M-Ebtehaj Jan-2011, Rev00

% Controlling the Inputs
if nargin==5, option=0;end
if nargin~=5 && nargin~=6 , error('Warning: Number of Inputs Arguments is not corrcet!!'); end 
if mod(nhood_l,2)==0 || nhood_l<=1,error('size of the neighborhood shall be odd and greater than 1');end
if size(im_nex)~=4*size(im_trm), error('Input data are not properly registered'); end
if Nsc<2, error('Number of decomposition levels are inconsistent'); end
% Resize TRMM image
im_trm=imresize(im_trm,4,'nearest');

% Image padding for construction of the pyramids, size~2^Nsc
pad_size1=2^Nsc*ceil(size(im_nex)/2^Nsc)-size(im_nex);
im_nex_p1=padarray(im_nex,pad_size1,'symmetric','post');
im_trm_p1=padarray(im_trm,pad_size1,'symmetric','post');

% The following pyramids include "Nsc" levels of undecimated H-V-D subbands
pyr_im_nex=swt2(im_nex_p1,Nsc,'haar');
if Nsc>2, pyr_im_trm=swt2(im_trm_p1,Nsc-2,'haar'); else pyr_im_trm=im_trm_p1; end
swt2_rec=zeros([size(im_nex_p1) 3*Nsc+1]);
swt2_rec(:,:,1:end-1)=pyr_im_nex(:,:,1:end-1);

% Padding the pyramid to handle nlfilering boundary issues
pad_size2=(nhood_l-1)/2*ones(1,2);
pyr_im_nex_l=padarray(pyr_im_nex(:,:,end),pad_size2,'symmetric','both');   % Low-pass Coef.
pyr_im_trm_l=4*padarray(pyr_im_trm(:,:,end),pad_size2,'symmetric','both'); % Low-pass Coef.
X_col  = im2col(pyr_im_trm_l,[nhood_l nhood_l],'sliding');
Y_col  = im2col(pyr_im_nex_l,[nhood_l nhood_l],'sliding');
X_col_cen=mean(X_col);
Y_col_cen=Y_col((nhood_l^2+1)/2,:);

% Projection of low-pass subband of the TRMM on the NEXRAD linear subspace
% x_low_hat=X_col_cen+(var(Y_col)./(var(Y_col)+sig_w^2)).*(Y_col_cen-X_col_cen);
x_low_hat=X_col_cen+(max((var(Y_col)-sig_w^2),0)./(max(var(Y_col),sig_w^2))).*(Y_col_cen-X_col_cen);

swt2_rec(:,:,end)=col2im(x_low_hat,[nhood_l nhood_l],size(pyr_im_nex_l));
im_fused = iswt2(swt2_rec,'haar');

% Returning back the Pyramid to its original size
im_fused=im_fused(1:size(im_fused,1)-pad_size1(1),1:size(im_fused,2)-pad_size1(2));
im_fused(im_fused<0)=0;
if option==0
    mask1=im_fused<15;
    mask2=(im_fused>0)&(im_nex==0);
    mask=(mask1)&(~mask2);
    im_fused(mask)=im_nex(mask);
else
    mask=im_fused<17;
    im_fused(mask)=im_nex(mask);
end  
disp('100% is Done!')
disp('Fusion is Completed!!')