function [sub_denos]=wiener_gsm(I,noise,nhood,prnt,Tol_z)
% I : is the MxNx2 array which contains two consecutive wavelet subbands of image
% MxN:  Size of the subband
% nhood: represents the size of a square neighboorhood, default=3
% noise: is a NxNx2 array which contains two consecutive subbands of the noise
% prnt: 1=> consider a parent; 0=> no parent, default=1
% Tol_z: is the stopping criteria for bisection estimation of z_hat, default=10^-4
if nargin<5, Tol_z=10^-4; end 
if nargin<4, prnt=1; end 
if nargin<3, nhood=3; end 
if nargin<2, error('Not enough Inputs! " Noise subbands must be defined" '); end ;
if prnt~=0 && prnt~=1, 
    prnt=1;
    disp('Warning: This version of the code uses prnt=0|1 ');
    disp('prnt is set to 1');
end

I_4std=I(:,:,1);
if std2(noise(:,:,1))> 0.9*std2(I_4std(I_4std>0))
    error('Signal is not dectable at this level of noise');
end

% Rearranging image blocks into columns
I_col  = im2col(I(:,:,1),[nhood nhood],'sliding');
I_c_col= im2col(I(:,:,2),[nhood nhood],'sliding');
% picking the central value of the parent(subband)
I_c_col=I_c_col((nhood^2+1)/2,:);
if prnt,
    y=[I_col;I_c_col];
else
    y=I_col;
end

% Rearranging noise blocks into columns
nois_col  = im2col(noise(:,:,1),[nhood nhood],'sliding');
nois_c_col= im2col(noise(:,:,2),[nhood nhood],'sliding');
% picking the central value of the parent(niose)
nois_c_col=nois_c_col((nhood^2+1)/2,:);
if prnt,
    nois_y=[nois_col;nois_c_col];
else
    nois_y=nois_col;
end

% Covaraince estimation of the subband coefficients
% Note that the covariance of those blocks which are filled entierly with
% zero intensity values are excluded from the estimation process
ind=(sum(y)>0); y_s=y(:,ind);
C_y=cov(y_s');
C_w=cov(nois_y');

% Duouble Checking the admissibility of the noise covaraince 
if mean(diag(C_w))>=mean(diag(C_y))
    disp('Warning!!: Signal is not detectable at this noise level');
    error(['The Maximum recommended standard deviation of the noise is: ',num2str(0.9*mean(diag(C_y)))]);
end
    
% Estimation of the log(z) parameters
sig2_z=(mean2(y.^4)/3-std2(y)^4)/(std2(y)^2-std2(nois_y)^2)^2; 
if sig2_z<=0, error('sig2_z is not positive definite'); end
mu_l=-log(sig2_z+1)/2;
sig2_l=log(sig2_z+1);

% Computing C_u
C_u=C_y-C_w;
[Q,L] = eig(C_u);
L = diag(diag(L).*(diag(L)>0))*sum(diag(L))/(sum(diag(L).*(diag(L)>0))+(sum(diag(L).*(diag(L)>0))==0));
C_u = Q*L/Q;

% Computing S=C_w^(1/2)
[Vw,dd] = eig(C_w);
S = Vw*real(sqrt(dd))*Vw'; % S*S' = C_w
iS = pinv(S);
if ~isreal(iS), error('iS is not real'); end

[Q,L] = eig(iS*C_u*iS');	 	  % Double diagonalization of signal and noise
lamb = diag(L);					  % eigenvalues: energy in the new represetnation.
lamb = real(lamb).*(real(lamb)>0); 
lamb(lamb==0)=eps;

% Linear transformation of the observations, and keep the quadratic values 
V = real(Q)'*iS*y;    
zn = (V.^2-1)./repmat(lamb,1,length(V));
lamb_vec_i=repmat(lamb.^-1,1,size(zn,2));

% Estimation of z
% Initialization of the bisection method
b_ub=50; pos_m=1;
a = zeros(1,length(zn))+eps;  a_vec=repmat(a,size(zn,1),1);
f_cost_a=(log(a)-mu_l+sig2_l)./(a*sig2_l) + sum((a_vec-zn)./(a_vec+lamb_vec_i).^2);
while pos_m~=0
    b = b_ub*ones(1,length(zn));   b_vec=repmat(b,size(zn,1),1);
    f_cost_b=(log(b)-mu_l+sig2_l)./(b*sig2_l) + sum((b_vec-zn)./(b_vec+lamb_vec_i).^2);
    pos_m=sum((f_cost_a<0)-(f_cost_b>0));
    b_ub=b_ub+50;
    if b_ub==10000, error('Upper and lower bound of the bisection method do not return different signs'); end 
end
while max(abs(b-a)) > Tol_z
    z_hat = (a+b)/2;
    z_vec=repmat(z_hat,size(zn,1),1); %vectorized Z
    f_cost=(log(z_hat)-mu_l+sig2_l)./(z_hat*sig2_l) + sum((z_vec-zn)./(z_vec+lamb_vec_i).^2);    
    a(f_cost<0)=z_hat(f_cost<0);
    b(f_cost>0)=z_hat(f_cost>0);
end
z_hat=(a+b)/2; 
z_hat=z_hat./mean(z_hat); % regularization of the estimated multiplier
M = S*Q;
x_hat=sum(repmat(M((nhood^2+1)/2,:)',1,size(zn,2)).*(V./(1+repmat(z_hat.^-1,size(zn,1),1).*lamb_vec_i)));
sub_denos=col2im(x_hat,[nhood nhood],[size(I,1) size(I,2)]);


